/*
  mk_vortex.C
*/

void r2xy(float r, float theta, float& x, float& y, 
	  const float x1, const float y1, const float x2, const float y2, 
	  const float rmax, const float rmin=0.1) {
  float theta1 = std::atan2(y1, x1);
  float theta2 = std::atan2(y1, x2);
  float theta3 = std::atan2(y2, x2);
  float theta4 = std::atan2(y2, x1);

  float x0 = rmax*std::cos(theta);
  float y0 = rmax*std::sin(theta);
  float theta0 = std::atan2(y0, x0);
  float xb=0.0;
  float yb=0.0;

  x = 0.0;
  y = 0.0;

  float tantheta = std::tan(theta);
  if (theta0 < theta1) {
    xb = x1;
    yb = xb*std::tan(theta);
  } else if (theta0 < theta2) {
    yb = y1;
    xb = yb/std::tan(theta);
  } else if (theta0 < theta3) {
    xb = x2;
    yb = xb*std::tan(theta);
  } else if (theta0 < theta4) {
    yb = y2;
    xb = yb/std::tan(theta);
  }
  //  float rb = std::sqrt(xb*xb + yb*yb);
  float s = std::log(r/rmin)/std::log(rmax/rmin);
  x = xb*s;
  y = yb*s;
}

void xy2r(float x, float y, float& r, float& theta, 
	  const float x1, const float y1, const float x2, const float y2, 
	  const float rmax, const float rmin=0.1) {
  float theta1 = std::atan2(y1, x1);
  float theta2 = std::atan2(y1, x2);
  float theta3 = std::atan2(y2, x2);
  float theta4 = std::atan2(y2, x1);

  float xb=0.0;
  float yb=0.0;

  r = 0.0;
  theta = std::atan2(y, x);
  
  float tantheta = std::tan(theta);
  if (theta < theta1) {
    xb = x1;
    yb = xb*std::tan(theta);
  } else if (theta < theta2) {
    yb = y1;
    xb = yb/std::tan(theta);
  } else if (theta < theta3) {
    xb = x2;
    yb = xb*std::tan(theta);
  } else if (theta < theta4) {
    yb = y2;
    xb = yb/std::tan(theta);
  }
  float rb = std::sqrt(xb*xb + yb*yb);
  float rxy = std::sqrt(x*x + y*y);
  // rxy = rb*log(r/rmin)/log(rmax/rmin);
  // log(r/rmin) = rxy/rb*log(rmax/rmin)
  float s = rxy/rb*std::log(rmax/rmin);
  r = rmin*std::exp(s);
}

float velocityAtR(float r) {
  float v1 = 50.0;
  float v = 0.0;
  float r1 = 5.0;
  if (r < r1) {
    float rr = r/r1;
    v = v1*rr*rr;
    v = 0.0;
  } else {
    v = v1/std::sqrt(r/r1);
  }
  return v;
}

void mk_vortex(const std::string& fout="work/vortex_ex1.root") {
  TFile* f = TFile::Open(fout.c_str(), "RECREATE");
  
  TH2F* h1x = new TH2F("h1x", "Single vortex vx", 100, -100, 100, 100, -100, 100);
  TH2F* h1y = new TH2F("h1y", "Single vortex vy", 100, -100, 100, 100, -100, 100);
  TH2F* h2x = new TH2F("h2x", "Two vortices vx", 200, -200, 200, 100, -100, 100);
  TH2F* h2y = new TH2F("h2y", "Two vortices vy", 200, -200, 200, 100, -100, 100);

  int nx = h1x->GetNbinsX();
  int ny = h1x->GetNbinsY();
  int ix, iy;
  float x, y, r, v, theta;
  float vx, vy;
  TAxis* xaxis=0;
  TAxis* yaxis=0;
  const float pi2 = TMath::Pi()/2.0;
  
  xaxis = h1x->GetXaxis();
  yaxis = h1x->GetYaxis();
  for (ix=1; ix<=nx; ++ix) {
    for (iy=1; iy<=ny; ++iy) {
      x = xaxis->GetBinCenter(ix);
      y = yaxis->GetBinCenter(iy);
      r = std::sqrt(x*x + y*y);
      theta = std::atan2(y, x);
      v = velocityAtR(r);
      vx = v*std::cos(theta+pi2);
      vy = v*std::sin(theta+pi2);
      h1x->SetBinContent(ix, iy, vx);
      h1y->SetBinContent(ix, iy, vy);
    }
  }

  float x0a=-100;
  float y0a=0;
  float x0b=100;
  float y0b=0;
  float dx, dy;
  float vxa, vya, vxb, vyb;

  xaxis = h2x->GetXaxis();
  yaxis = h2x->GetYaxis();
  nx = xaxis->GetNbins();
  ny = yaxis->GetNbins();
  for (ix=1; ix<=nx; ++ix) {
    for (iy=1; iy<=ny; ++iy) {
      x = xaxis->GetBinCenter(ix);
      y = yaxis->GetBinCenter(iy);
      // Vector from vortex A
      dx = x-x0a;
      dy = y-y0a;
      r = std::sqrt(dx*dx + dy*dy);
      theta = std::atan2(dy, dx);
      v = velocityAtR(r);
      vxa = v*std::cos(theta+pi2);
      vya = v*std::sin(theta+pi2);
      // Vector from vortex B
      dx = x-x0b;
      dy = y-y0b;
      r = std::sqrt(dx*dx + dy*dy);
      theta = std::atan2(dy, dx);
      v = velocityAtR(r);
      vxb = -v*std::cos(theta+pi2);
      vyb = -v*std::sin(theta+pi2);
      // Add contribution from two centers
      vx = vxa + vxb;
      vy = vya + vyb;
      h2x->SetBinContent(ix, iy, vx);
      h2y->SetBinContent(ix, iy, vy);
    }
  }

  f->Write();
}
