/*
  pl_vec.C
*/

void drawArrow(float x, float y, float vx, float vy, float dscale) {
  char opt[20]=">";
  float asize=0.01;
  float x2=x+vx*dscale;
  float y2=y+vy*dscale;
  TArrow* arrow = new TArrow(x, y, x2, y2, asize, opt);
  arrow->SetLineWidth(1);
  arrow->SetAngle(40);
  arrow->Draw();
}

void set_style() {
  gStyle->SetOptStat(0);
  gStyle->SetPadTopMargin(0.03);
  gStyle->SetPadBottomMargin(0.11);
  gStyle->SetPadLeftMargin(0.11);
  gStyle->SetPadRightMargin(0.08);
  gStyle->SetNdivisions(505, "XY");
  gStyle->SetTitleSize(0.05, "XY");
  gStyle->SetLabelSize(0.05, "XY");
  gStyle->SetTitleOffset(1.0, "X");
  gStyle->SetTitleOffset(1.0, "Y");
}

TH2* velocityHist(TH2* hx, TH2* hy) {
  char hname[200];
  std::sprintf(hname, "%s_v", hx->GetName() );
  int ix, iy;
  float vx, vy, v;
  float omega;
  
  TH2* h = dynamic_cast<TH2*>(hx->Clone(hname) );
  h->Reset();
  int nx = h->GetNbinsX();
  int ny = h->GetNbinsY();

  for (ix=1; ix<=nx; ++ix) {
    for (iy=1; iy<=ny; ++iy) {
      vx = hx->GetBinContent(ix, iy);
      vy = hy->GetBinContent(ix, iy);
      v = std::sqrt(vx*vx + vy*vy);
      h->SetBinContent(ix, iy, v);
    }
  }
  return h;
}
  
void pl_vec(TH2* hx, TH2* hy) {
  set_style();
  
  int ix, iy;
  int nx, ny;
  TAxis* xaxis = hx->GetXaxis();
  TAxis* yaxis = hy->GetYaxis();
  nx = xaxis->GetNbins();
  ny = yaxis->GetNbins();
  int nstep=5;
  float x, y;
  float vx, vy;
  float dscale=0.5;
  
  char hname[200];
  std::sprintf(hname, "%s_vec", hx->GetName() );

  float x1 = xaxis->GetBinLowEdge(1);
  float x2 = xaxis->GetBinUpEdge(nx);
  float y1 = yaxis->GetBinLowEdge(1);
  float y2 = yaxis->GetBinUpEdge(ny);
  TCanvas* c = new TCanvas("c", "", 0, 0, 600, 400);
  TH1* hframe = c->DrawFrame(x1, y1, x2, y2);

  TH2* h_v = velocityHist(hx, hy);
  h_v->Draw("same zcol");
  
  for (ix=1; ix<nx; ix+=nstep) {
    for (iy=1; iy<ny; iy+=nstep) {
      vx = hx->GetBinContent(ix, iy);
      vy = hy->GetBinContent(ix, iy);
      x = xaxis->GetBinCenter(ix);
      y = yaxis->GetBinCenter(iy);
      drawArrow(x, y, vx, vy, dscale);
    }
  }
  
  hframe->SetXTitle("X");
  hframe->SetYTitle("Y");
  c->Print("figures/vec.png");
}

void pl_vec(const std::string& fname="work/vortex_ex1.root",
	    const std::string& hnamex="h2x",
	    const std::string& hnamey="h2y") {
  TFile* f = TFile::Open(fname.c_str(), "READ");
  TH2* hx = dynamic_cast<TH2*>(f->Get(hnamex.c_str() ) );
  TH2* hy = dynamic_cast<TH2*>(f->Get(hnamey.c_str() ) );

  pl_vec(hx, hy);
}
